@@ -22,20 +22,26 @@ import java.util.HashMap;
 import java.util.List;
 
 import javax.enterprise.inject.Instance;
+import javax.ws.rs.core.Response;
 
 import org.apache.commons.io.IOUtils;
 import org.jbpm.workbench.forms.display.FormRenderingSettings;
+import org.jbpm.workbench.forms.display.api.KieServerFormRenderingSettings;
 import org.jbpm.workbench.forms.display.api.KieWorkbenchFormRenderingSettings;
+import org.jbpm.workbench.forms.display.api.TaskFormPermissionDeniedException;
 import org.jbpm.workbench.forms.display.backend.provider.DefaultKieWorkbenchFormsProvider;
 import org.jbpm.workbench.forms.display.backend.provider.KieWorkbenchFormsProvider;
 import org.jbpm.workbench.forms.display.backend.provider.ProcessFormsValuesProcessor;
 import org.jbpm.workbench.forms.display.backend.provider.TaskFormValuesProcessor;
 import org.jbpm.workbench.forms.display.backend.provider.util.FormContentReader;
 import org.jbpm.workbench.forms.service.providing.FormProvider;
+import org.jbpm.workbench.forms.service.providing.ProcessRenderingSettings;
+import org.jbpm.workbench.forms.service.providing.TaskRenderingSettings;
 import org.junit.Before;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.kie.server.api.exception.KieServicesException;
+import org.kie.server.api.exception.KieServicesHttpException;
 import org.kie.server.api.model.definition.ProcessDefinition;
 import org.kie.server.api.model.definition.TaskInputsDefinition;
 import org.kie.server.api.model.definition.TaskOutputsDefinition;
@@ -46,19 +52,24 @@ import org.kie.server.client.ProcessServicesClient;
 import org.kie.server.client.UIServicesClient;
 import org.kie.server.client.UserTaskServicesClient;
 import org.kie.soup.project.datamodel.commons.util.RawMVELEvaluator;
+import org.kie.workbench.common.forms.data.modeller.service.ext.ModelReaderService;
+import org.kie.workbench.common.forms.data.modeller.service.impl.ext.dmo.runtime.RuntimeDMOModelReader;
 import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.BackendFormRenderingContextManagerImpl;
-import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.FormValuesProcessorImpl;
-import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.fieldProcessors.MultipleSubFormFieldValueProcessor;
-import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.fieldProcessors.SubFormFieldValueProcessor;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.FieldValueMarshaller;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.FieldValueMarshallerRegistry;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.FieldValueMarshallerRegistryImpl;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.TextAreaFormFieldValueMarshaller;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.models.MultipleSubFormFieldValueMarshaller;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.models.SubFormFieldValueMarshaller;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.time.DateMultipleInputFieldValueMarshaller;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.time.DateMultipleSelectorFieldValueMarshaller;
+import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.impl.marshalling.time.LocalDateFieldValueMarshaller;
 import org.kie.workbench.common.forms.dynamic.backend.server.context.generation.dynamic.validation.impl.ContextModelConstraintsExtractorImpl;
-import org.kie.workbench.common.forms.dynamic.service.context.generation.dynamic.FieldValueProcessor;
-import org.kie.workbench.common.forms.dynamic.service.context.generation.dynamic.FormValuesProcessor;
 import org.kie.workbench.common.forms.fields.test.TestFieldManager;
 import org.kie.workbench.common.forms.fields.test.TestMetaDataEntryManager;
 import org.kie.workbench.common.forms.jbpm.server.service.formGeneration.impl.runtime.BPMNRuntimeFormGeneratorService;
 import org.kie.workbench.common.forms.jbpm.server.service.impl.DynamicBPMNFormGeneratorImpl;
 import org.kie.workbench.common.forms.jbpm.service.bpmn.DynamicBPMNFormGenerator;
-import org.kie.workbench.common.forms.model.FieldDefinition;
 import org.kie.workbench.common.forms.services.backend.serialization.FormDefinitionSerializer;
 import org.kie.workbench.common.forms.services.backend.serialization.impl.FieldSerializer;
 import org.kie.workbench.common.forms.services.backend.serialization.impl.FormDefinitionSerializerImpl;
@@ -66,8 +77,19 @@ import org.kie.workbench.common.forms.services.backend.serialization.impl.FormMo
 import org.mockito.Mock;
 import org.mockito.runners.MockitoJUnitRunner;
 
-import static org.junit.Assert.*;
-import static org.mockito.Mockito.*;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+import static org.mockito.Mockito.any;
+import static org.mockito.Mockito.anyBoolean;
+import static org.mockito.Mockito.anyLong;
+import static org.mockito.Mockito.anyString;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.never;
+import static org.mockito.Mockito.spy;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
+import static org.mockito.Mockito.when;
 
 @RunWith(MockitoJUnitRunner.class)
 public class FormServiceEntryPointImplTest {
@@ -91,7 +113,10 @@ public class FormServiceEntryPointImplTest {
     @Mock
     private KieServicesClient kieServicesClient;
 
-    private FormValuesProcessor formValuesProcessor;
+    @Mock
+    private ModelReaderService<ClassLoader> modelReaderService;
+
+    private FieldValueMarshallerRegistry registry;
 
     private DynamicBPMNFormGenerator dynamicBPMNFormGenerator;
 
@@ -105,21 +130,31 @@ public class FormServiceEntryPointImplTest {
 
     @Before
     public void init() {
-        List<FieldValueProcessor> processors = Arrays.asList(new SubFormFieldValueProcessor(),
-                                                             new MultipleSubFormFieldValueProcessor());
+        SubFormFieldValueMarshaller subFormFieldValueMarshaller = new SubFormFieldValueMarshaller();
+        MultipleSubFormFieldValueMarshaller multipleSubFormFieldValueMarshaller = new MultipleSubFormFieldValueMarshaller();
 
-        Instance<FieldValueProcessor<? extends FieldDefinition, ?, ?>> fieldValueProcessors = mock(Instance.class);
-        when(fieldValueProcessors.iterator()).then(proc -> processors.iterator());
+        List<FieldValueMarshaller> marshallers = Arrays.asList(subFormFieldValueMarshaller,
+                                                               multipleSubFormFieldValueMarshaller,
+                                                               new DateMultipleInputFieldValueMarshaller(),
+                                                               new DateMultipleSelectorFieldValueMarshaller(),
+                                                               new LocalDateFieldValueMarshaller(),
+                                                               new TextAreaFormFieldValueMarshaller());
 
-        formValuesProcessor = new FormValuesProcessorImpl(fieldValueProcessors);
+        Instance<FieldValueMarshaller<?, ?, ?>> marshallersInstance = mock(Instance.class);
 
-        Instance<FormProvider<? extends FormRenderingSettings>> instance = mock(Instance.class);
+        when(marshallersInstance.iterator()).then(proc -> marshallers.iterator());
 
-        backendFormRenderingContextManager = new BackendFormRenderingContextManagerImpl(formValuesProcessor,
-                                                                                        new ContextModelConstraintsExtractorImpl());
+        registry = new FieldValueMarshallerRegistryImpl(marshallersInstance);
 
-        runtimeFormGeneratorService = new BPMNRuntimeFormGeneratorService(new TestFieldManager(),
-                                                                          new RawMVELEvaluator());
+        subFormFieldValueMarshaller.setRegistry(registry);
+
+        multipleSubFormFieldValueMarshaller.setRegistry(registry);
+
+        backendFormRenderingContextManager = new BackendFormRenderingContextManagerImpl(registry, new ContextModelConstraintsExtractorImpl());
+
+        when(modelReaderService.getModelReader(any())).thenReturn(new RuntimeDMOModelReader(this.getClass().getClassLoader(), new RawMVELEvaluator()));
+
+        runtimeFormGeneratorService = new BPMNRuntimeFormGeneratorService(modelReaderService, new TestFieldManager());
 
         dynamicBPMNFormGenerator = new DynamicBPMNFormGeneratorImpl(runtimeFormGeneratorService);
 
@@ -137,15 +172,15 @@ public class FormServiceEntryPointImplTest {
         kieWorkbenchFormsProvider = new KieWorkbenchFormsProvider(processValuesProcessor,
                                                                   taskValuesProcessor);
 
-        defaultProvider = new DefaultKieWorkbenchFormsProvider(processValuesProcessor,
-                                                               taskValuesProcessor);
+        defaultProvider = spy(new DefaultKieWorkbenchFormsProvider(processValuesProcessor, taskValuesProcessor));
 
         when(kieServicesClient.getClassLoader()).thenReturn(this.getClass().getClassLoader());
 
+        Instance<FormProvider<? extends FormRenderingSettings>> instance = mock(Instance.class);
+
         when(instance.iterator()).then(result -> Arrays.asList(kieWorkbenchFormsProvider).iterator());
 
-        serviceEntryPoint = new FormServiceEntryPointImpl(instance,
-                                                          defaultProvider) {
+        serviceEntryPoint = new FormServiceEntryPointImpl(instance, defaultProvider) {
 
             @Override
             protected <T> T getClient(String serverTemplateId,
@@ -216,7 +251,8 @@ public class FormServiceEntryPointImplTest {
 
         FormRenderingSettings settings = serviceEntryPoint.getFormDisplayProcess("template",
                                                                                  "domain",
-                                                                                 "invoices");
+                                                                                 "invoices",
+                                                                                 false);
 
         verify(processServicesClient).getProcessDefinition(anyString(),
                                                            anyString());
@@ -224,6 +260,8 @@ public class FormServiceEntryPointImplTest {
         verify(uiServicesClient).getProcessRawForm(anyString(),
                                                    anyString());
 
+        verify(defaultProvider, never()).render(any(ProcessRenderingSettings.class));
+
         checkRenderingSettings(settings);
     }
 
@@ -232,7 +270,8 @@ public class FormServiceEntryPointImplTest {
 
         FormRenderingSettings settings = serviceEntryPoint.getFormDisplayProcess("template",
                                                                                  "domain",
-                                                                                 "invoices");
+                                                                                 "invoices",
+                                                                                 false);
 
         verify(processServicesClient).getProcessDefinition(anyString(),
                                                            anyString());
@@ -241,6 +280,8 @@ public class FormServiceEntryPointImplTest {
         verify(uiServicesClient).getProcessRawForm(anyString(),
                                                    anyString());
 
+        verify(defaultProvider).render(any(ProcessRenderingSettings.class));
+
         checkRenderingSettings(settings);
     }
 
@@ -254,7 +295,8 @@ public class FormServiceEntryPointImplTest {
 
         FormRenderingSettings settings = serviceEntryPoint.getFormDisplayProcess("template",
                                                                                  "domain",
-                                                                                 "invoices");
+                                                                                 "invoices",
+                                                                                 false);
 
         verify(processServicesClient).getProcessDefinition(anyString(),
                                                            anyString());
@@ -263,6 +305,8 @@ public class FormServiceEntryPointImplTest {
         verify(uiServicesClient).getProcessRawForm(anyString(),
                                                    anyString());
 
+        verify(defaultProvider).render(any(ProcessRenderingSettings.class));
+
         checkRenderingSettings(settings);
     }
 
@@ -274,7 +318,8 @@ public class FormServiceEntryPointImplTest {
 
         FormRenderingSettings settings = serviceEntryPoint.getFormDisplayProcess("template",
                                                                                  "domain",
-                                                                                 "invoices");
+                                                                                 "invoices",
+                                                                                 false);
 
         verify(processServicesClient).getProcessDefinition(anyString(),
                                                            anyString());
@@ -282,6 +327,8 @@ public class FormServiceEntryPointImplTest {
         verify(uiServicesClient).getProcessRawForm(anyString(),
                                                    anyString());
 
+        verify(defaultProvider).render(any(ProcessRenderingSettings.class));
+
         checkRenderingSettings(settings);
     }
 
@@ -306,6 +353,8 @@ public class FormServiceEntryPointImplTest {
         verify(uiServicesClient).getTaskRawForm(anyString(),
                                                 anyLong());
 
+        verify(defaultProvider, never()).render(any(TaskRenderingSettings.class));
+
         checkRenderingSettings(settings);
     }
 
@@ -322,16 +371,65 @@ public class FormServiceEntryPointImplTest {
                                                        anyBoolean());
         verify(kieServicesClient,
                times(2)).getClassLoader();
+
+        verify(uiServicesClient).getTaskRawForm(anyString(),
+                                                anyLong());
+
+        verify(defaultProvider).render(any(TaskRenderingSettings.class));
+
+        checkRenderingSettings(settings);
+    }
+
+    @Test(expected = TaskFormPermissionDeniedException.class)
+    public void testRenderTaskFormWithPermissionException() {
+        when(uiServicesClient.getTaskRawForm(anyString(), anyLong()))
+                .thenThrow(new KieServicesHttpException("", Response.Status.UNAUTHORIZED.getStatusCode(), "", ""));
+
+        FormRenderingSettings settings = serviceEntryPoint.getFormDisplayTask("template",
+                                                                              "domain",
+                                                                              12);
+
+        verify(userTaskServicesClient).getTaskInstance(anyString(),
+                                                       anyLong(),
+                                                       anyBoolean(),
+                                                       anyBoolean(),
+                                                       anyBoolean());
+        verify(kieServicesClient,
+               times(2)).getClassLoader();
+
         verify(uiServicesClient).getTaskRawForm(anyString(),
                                                 anyLong());
 
         checkRenderingSettings(settings);
     }
 
+    @Test
+    public void testRenderTaskFormWithException() {
+        when(uiServicesClient.getTaskRawForm(anyString(), anyLong())).thenThrow(new RuntimeException("Unable to find form"));
+
+        FormRenderingSettings settings = serviceEntryPoint.getFormDisplayTask("template",
+                                                                              "domain",
+                                                                              12);
+
+        verify(userTaskServicesClient).getTaskInstance(anyString(),
+                                                       anyLong(),
+                                                       anyBoolean(),
+                                                       anyBoolean(),
+                                                       anyBoolean());
+
+        verify(kieServicesClient).getClassLoader();
+
+        verify(uiServicesClient).getTaskRawForm(anyString(),
+                                                anyLong());
+
+        verify(defaultProvider).render(any(TaskRenderingSettings.class));
+
+        checkRenderingSettings(settings);
+    }
+
     @Test
     public void testRenderTaskDefaultFormWithException() {
-        when(uiServicesClient.getTaskRawForm(anyString(),
-                                             anyLong())).thenThrow(new KieServicesException("Unable to find form"));
+        when(uiServicesClient.getTaskRawForm(anyString(), anyLong())).thenThrow(new KieServicesException("Unable to find form"));
 
         FormRenderingSettings settings = serviceEntryPoint.getFormDisplayTask("template",
                                                                               "domain",
@@ -346,6 +444,8 @@ public class FormServiceEntryPointImplTest {
         verify(uiServicesClient).getTaskRawForm(anyString(),
                                                 anyLong());
 
+        verify(defaultProvider).render(any(TaskRenderingSettings.class));
+
         checkRenderingSettings(settings);
     }
 
@@ -373,6 +473,47 @@ public class FormServiceEntryPointImplTest {
 
         checkRenderingSettings(settings);
     }
+    
+    @Test
+    public void testKieServerRenderProcessForm() {
+
+        serviceEntryPoint.setKieServerFormRenderer(true); 
+        FormRenderingSettings settings = serviceEntryPoint.getFormDisplayProcess("template",
+                                                                                 "domain",
+                                                                                 "invoices",
+                                                                                 false);
+
+        verify(processServicesClient, never()).getProcessDefinition(anyString(),
+                                                           anyString());
+        verify(kieServicesClient, never()).getClassLoader();
+        verify(uiServicesClient, never()).getProcessRawForm(anyString(),
+                                                   anyString());
+
+        verify(defaultProvider, never()).render(any(ProcessRenderingSettings.class));
+
+        checkKieServerRenderingSettings(settings);
+    }
+    
+    @Test
+    public void testKieServerRenderTaskForm() {
+        serviceEntryPoint.setKieServerFormRenderer(true); 
+        FormRenderingSettings settings = serviceEntryPoint.getFormDisplayTask("template",
+                                                                              "domain",
+                                                                              12);
+
+        verify(userTaskServicesClient, never()).getTaskInstance(anyString(),
+                                                       anyLong(),
+                                                       anyBoolean(),
+                                                       anyBoolean(),
+                                                       anyBoolean());
+        verify(kieServicesClient, never()).getClassLoader();
+        verify(uiServicesClient, never()).getTaskRawForm(anyString(),
+                                                anyLong());
+
+        verify(defaultProvider, never()).render(any(TaskRenderingSettings.class));
+
+        checkKieServerRenderingSettings(settings);
+    }
 
     protected void checkRenderingSettings(FormRenderingSettings settings) {
         assertNotNull("Settings cannot be null",
@@ -388,6 +529,18 @@ public class FormServiceEntryPointImplTest {
         assertNotNull("There should be a default FormDefinition",
                       wbSettings.getRenderingContext().getRootForm());
     }
+    
+    protected void checkKieServerRenderingSettings(FormRenderingSettings settings) {
+        assertNotNull("Settings cannot be null",
+                      settings);
+        assertTrue("Settings must be KieServer Forms",
+                   settings instanceof KieServerFormRenderingSettings);
+
+        KieServerFormRenderingSettings ksSettings = (KieServerFormRenderingSettings) settings;
+
+        assertNotNull("URL shouldn't be empty",
+                      ksSettings.getUrl());
+    }
 
     protected String getFormContent() {
         try {
